#ifndef __BSEARCH_H
#define __BSEARCH_H

/*
	purpose: to use the binar search algorithm to search
		 through a pre-sorted container for a given
		 type. The class C of the template relates to the
		 container class type (i.e. Vector< int >) which
		 must come equiped with the [] overloaded operators.
		 The second argument is the type to search for and
		 its related value ( note: it may be an attribute
		 of the type or the type) and the third argument is
		 a ptr to a function that excepts two occurences
		 of type T and returns -1 for t1 < t2, 0 for t1 == t2
		 and 1 for t1 > t2.

	returns: the index along the container type where the match
		 was found or -1 upon failure to find a match.
*/

template< class Size_t, class C, class S, class T > long bsearch( C& v,
S x, Size_t size, signed int (*compare)( S t1, T t2 ) )
{
Size_t	low, high, mid;

	low = 0;
	high = size - 1;

	while( low <= high ) {
		mid = (low + high) / 2;

		if( compare( x, v[mid] ) < 0 )
			high = mid - 1;
		else
		if( compare( x, v[mid] ) > 0 )
			low = mid + 1;
		else
			return mid;
	}
	return -1;
}

/*
Example 1: (Using pointers to a type).

class	Integer {
		int i;
	public:
		Integer( int i ) { this->i = i; }
	int	getInteger() const
			{ return i; }
};

int compare( int t1, Integer* t2 )
{
	if( t1 < t2->getInteger() )
		return -1;
	if( t1 == t2->getInteger() )
		return 0;

	return 1;
}

int compare2( Integer* t1, Integer* t2 )
{
	if( t1->getInteger() < t2->getInteger() )
		return -1;
	if( t1->getInteger() == t2->getInteger() )
		return 0;

	return 1;
}

void	test( Vector< Integer* >& v )
{
	for( int i = v.sizeOf() - 1; i >= 0; i-- )
		v[i] = new Integer( rand() );

	bubblesort( v, compare2 );

	cout << "BSearch item found index "
	     << bsearch( v, 7117, compare ) << endl;

	for( i = 0; i < v.sizeOf(); i++ )
		cout << v[i]->getInteger() << endl;
}

void	main()
{
Vector< Integer* > vec( 10 );

	test( v );
}

/---------------------------------------------------------------------------/

Example 2: (using constant/built in types)

Note: Ensure VECTOR_NO_DELETE is defined

int	compare( int t1, int t2 )
{
	if( t1 < t2 )
		return -1;	// t1 < t2
	if( t1 > t2 )
		return 1;	// t1 > t2

	return 0;		// t1 == t2
}

void	test( Vector< int >& vec )
{

	cout << "Prior to sorting" << endl;
	for( int i = vec.sizeOf() - 1; i >= 0; i-- ) {
	     vec[i] = rand();
	     cout << vec[i] << endl;
	}


	bubblesort( vec, compare );

	cout << "Item found at index " << bsearch( vec, 7117, compare ) << endl;

	cout << "Post sorting" << endl;
	for( int j = 0; j < vec.sizeOf(); j++ )
		cout << vec[j] << endl;

}

void	main()
{
Vector< int > vec( 10 );

	test( vec );
}

*/

#endif
